use super::File;
use std::path::PathBuf;
use byteorder::{ByteOrder, LittleEndian};
use tool_helper::{Error, format_if_error, read_file};

#[repr(packed)]
struct OverlayHeader {
    pub start_adr: u32,
    pub _lba_count: u16,
}

impl OverlayHeader {
    pub fn lba_count_offset() -> usize {
        let dummy = OverlayHeader{start_adr: 0, _lba_count: 0}.start_adr;
        std::mem::size_of_val(&dummy)
    }
}

pub fn load_from(file_name: &str, file_path: PathBuf) -> Result<File, Error> {
    let     overlay_header_size = std::mem::size_of::<OverlayHeader>();
    let mut content             = read_file(&file_path)?;

    if content.len() < overlay_header_size {
        return Err(Error::from_text(format!("Overlay {} has no header!", file_path.to_string_lossy())));
    }

    let lba_count       = LittleEndian::read_u16(&content[OverlayHeader::lba_count_offset()..]) as usize;
    let lba_header_size = lba_count*std::mem::size_of::<u32>();
    let lba_header      = &mut content[overlay_header_size..(overlay_header_size+lba_header_size)];

    if lba_header.is_empty() {
        return Err(Error::from_text(format!("LBA header of overlay {} is smaller then {} elements", file_path.to_string_lossy(), lba_count)));
    }

    let mut count = 0;
    for byte in lba_header {
        *byte = count as u8;
        count += 1;
    }

    let content_size = format_if_error!(tool_helper::compress::lz4(&content, 16), "Compressing {} failed with \"{error_text}\"", file_path.to_string_lossy())?.len();
    Ok(File::new_overlay(file_name, content, content_size)?)
}